@@ -1,21 +1,25 @@
 /*
  * gbsplay is a Gameboy sound player
  *
- * 2003-2005,2017 (C) by Christian Garbs <mitch@cgarbs.de>
- *                       Tobias Diedrich <ranma+gbsplay@tdiedrich.de>
- * Licensed under GNU GPL v1 or, at your option, any later version.
- *
  * header file for DirectSound output under Windows, aquanight <aquanight@gmail.com>
  * This is C++ because COM is a pain in pure C
+ *
+ * 2003-2020 (C) by Christian Garbs <mitch@cgarbs.de>
+ *                  Tobias Diedrich <ranma+gbsplay@tdiedrich.de>
+ *
+ * Licensed under GNU GPL v1 or, at your option, any later version.
  */
 
-#include <initguid.h>
-#include "common.h"
-#include "plugout.h"
-#include <windows.h>
-#include <dsound.h>
 #include <stdlib.h>
 #include <unistd.h>
+#include <assert.h>
+
+#include <initguid.h>
+#include <windows.h>
+#include <dsound.h>
+
+#include "common.h"
+#include "plugout.h"
 
 static LPDIRECTSOUND8 dsound_device;
 static LPDIRECTSOUNDBUFFER8 dsound_buffer;
@@ -26,15 +30,13 @@ static DWORD writeMax;       // Maximum amount we can write at once.
 static WAVEFORMATEX wfx;
 static DSBUFFERDESC dsbdesc;
 
-static long regparm dsound_open(enum plugout_endian endian, long rate)
+static long dsound_open(enum plugout_endian *endian, long rate, long *buffer_bytes, const char *const filename)
 {
 	HRESULT hr;
+	
+	UNUSED(filename);
 
-	if (endian != PLUGOUT_ENDIAN_NATIVE)
-	{
-		fprintf(stderr, "Currently only native endianness is supported.");
-		return -1;
-	}
+	*endian = PLUGOUT_ENDIAN_NATIVE;
 
 	hr = DirectSoundCreate8(NULL, &dsound_device, NULL);
 
@@ -72,6 +74,7 @@ static long regparm dsound_open(enum plugout_endian endian, long rate)
 			case DSERR_UNSUPPORTED:
 			case DSERR_UNINITIALIZED:
 				fprintf(stderr, "Wat. ");
+				break;
 			default:
 				fprintf(stderr, "SetCooperativeLevel failed: HRESULT = %lx\n", (unsigned long)hr);
 				break;
@@ -94,13 +97,15 @@ static long regparm dsound_open(enum plugout_endian endian, long rate)
 	memset(&dsbdesc, 0, sizeof(DSBUFFERDESC));
 	dsbdesc.dwSize = sizeof(DSBUFFERDESC);
 	dsbdesc.dwFlags = DSBCAPS_GLOBALFOCUS | DSBCAPS_GETCURRENTPOSITION2 | DSBCAPS_CTRLPAN | DSBCAPS_CTRLVOLUME | DSBCAPS_CTRLFREQUENCY; /* Keep sound buffer going if inactive, use accurate play cursor */
-	writeMax = dsbdesc.dwBufferBytes = wfx.nAvgBytesPerSec / 10;
+	/* Round off buffer size to sample boundary */
+	writeMax = dsbdesc.dwBufferBytes = (wfx.nAvgBytesPerSec / 10) & ~(wfx.nBlockAlign - 1);
 	dsbdesc.lpwfxFormat = &wfx;
 
+	*buffer_bytes = writeMax;
 	return 0; /* We're open, rock and roll! */
 }
 
-static ssize_t regparm dsound_write(const void* buf, size_t count)
+static ssize_t dsound_write(const void* buf, size_t count)
 {
 	size_t orig_count = count;
 	DWORD playCursor, writeCursor;
@@ -111,11 +116,9 @@ static ssize_t regparm dsound_write(const void* buf, size_t count)
 	while (count > 0)
 	{
 		IDirectSoundBuffer8_GetCurrentPosition(dsound_buffer, &playCursor, &writeCursor);
-		if (writeOffset == -1)
+		if (writeOffset == -1) {
+			/* Initial buffering */
 			writeOffset = writeCursor;
-		if (writeCursor == 0 && playCursor == 0)
-		{
-			/* Starting write before DSound begins playing */
 			writeLimit = writeMax - writeOffset;
 		}
 		else if (writeOffset <= playCursor)
@@ -136,14 +139,21 @@ static ssize_t regparm dsound_write(const void* buf, size_t count)
 			writeLimit = count;
 		}
 		IDirectSoundBuffer8_Lock(dsound_buffer, writeOffset, writeLimit, &pBuf1, &nBuf1, &pBuf2, &nBuf2, 0);
+		assert(pBuf1 != NULL);
 		memcpy(pBuf1, buf, nBuf1);
-		if (nBuf1 < writeLimit)
+		if (pBuf2)
 		{
+			/* DirectSound gave us a wraparound pointer because the requested lock
+			   extends past the end of the buffer */
 			memcpy(pBuf2, (void*)((char*)buf + nBuf1), nBuf2);
 			writeOffset = nBuf2; /* NOT += */
 		}
-		else
-			writeOffset += nBuf1;
+		else {
+		/* All requested data fits buffer without wraparound; make sure we don't
+		   start the next request right at the end of the buffer (reset to 0 instead) */
+			assert((writeOffset + nBuf1) <= writeMax);
+			writeOffset = (writeOffset + nBuf1) % writeMax;
+		}
 		IDirectSoundBuffer8_Unlock(dsound_buffer, pBuf1, nBuf1, pBuf2, nBuf2);
 		if (writeLimit < count)
 		{
@@ -157,7 +167,7 @@ static ssize_t regparm dsound_write(const void* buf, size_t count)
 	return orig_count;
 }
 
-static int regparm dsound_skip(int subsound)
+static int dsound_skip(int subsong)
 {
 	HRESULT hr;
 	LPDIRECTSOUNDBUFFER pDsb = NULL;
@@ -179,7 +189,7 @@ static int regparm dsound_skip(int subsound)
 	return 0;
 }
 
-static void regparm dsound_pause(int pause_mode)
+static void dsound_pause(int pause_mode)
 {
 	if (pause_mode && dsound_buffer)
 		IDirectSoundBuffer8_Stop(dsound_buffer);
@@ -187,7 +197,7 @@ static void regparm dsound_pause(int pause_mode)
 		IDirectSoundBuffer8_Play(dsound_buffer, 0, 0, DSBPLAY_LOOPING);
 }
 
-static void regparm dsound_close()
+static void dsound_close()
 {
 	if (dsound_buffer)
 	{
